package com.example.android.google.oauth.sample;

import android.accounts.AccountManager;
import android.content.Intent;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.view.Menu;
import android.view.MenuItem;

import com.example.android.google.oauth.sample.common.logger.Log;
import com.example.android.google.oauth.sample.common.logger.LogView;
import com.example.android.google.oauth.sample.common.logger.LogWrapper;
import com.example.android.google.oauth.sample.common.logger.MessageOnlyLogFilter;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.googleapis.extensions.android.gms.auth.GoogleAccountCredential;
import com.google.api.client.googleapis.extensions.android.gms.auth.UserRecoverableAuthIOException;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.gson.GsonFactory;
import com.google.api.services.drive.Drive;
import com.google.api.services.drive.DriveScopes;
import com.google.api.services.drive.model.File;
import com.google.api.services.drive.model.FileList;
import com.google.api.services.plus.Plus;
import com.google.api.services.plus.PlusScopes;
import com.google.api.services.plus.model.Person;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

public class MainActivity extends AppCompatActivity {
    private static final String TAG = "MainActivity";
    private static final int REQUEST_ACCOUNT_CHOOSER = 1;
    private static final int REQUEST_AUTHORIZATION_FROM_DRIVE = 2;
    private static final int REQUEST_AUTHORIZATION_FROM_PLUS = 3;

    private GoogleAccountCredential mCredential;
    private Drive mDrive;
    private Plus mPlus;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        initializeLogging();
        showAccountChooser();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        boolean isSelectedAccount = mCredential != null && mCredential.getSelectedAccountName() != null;
        menu.getItem(0).setEnabled(!isSelectedAccount);
        menu.getItem(1).setEnabled(isSelectedAccount);
        menu.getItem(2).setEnabled(isSelectedAccount);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        switch (id) {
            case R.id.action_oauth:
                Log.i(TAG, "--- Authentication ---");
                showAccountChooser();
                return true;
            case R.id.action_drive:
                Log.i(TAG, "--- List Drive files ---");
                listDriveFiles();
                return true;
            case R.id.action_plus:
                Log.i(TAG, "--- Show Plus user ---");
                showPlusUser();
                return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        switch (requestCode) {
            case REQUEST_ACCOUNT_CHOOSER:
                if (resultCode == RESULT_OK && data != null) {
                    String accountName = data.getStringExtra(AccountManager.KEY_ACCOUNT_NAME);
                    Log.i(TAG, "Account Name:" + accountName);
                    if (accountName != null) {
                        mCredential.setSelectedAccountName(accountName);
                    }
                }
                break;
            case REQUEST_AUTHORIZATION_FROM_DRIVE:
            case REQUEST_AUTHORIZATION_FROM_PLUS:
                if (resultCode == RESULT_OK) {
                    if (requestCode == REQUEST_AUTHORIZATION_FROM_DRIVE) {
                        listDriveFiles();
                    } else if (requestCode == REQUEST_AUTHORIZATION_FROM_PLUS) {
                        showPlusUser();
                    }
                }
                break;
        }
    }

    @NonNull
    private GoogleAccountCredential makeCredential() {
        Collection<String> scopes = Arrays.asList(
                DriveScopes.DRIVE_READONLY,
                PlusScopes.USERINFO_PROFILE);
        return GoogleAccountCredential.usingOAuth2(getApplicationContext(), scopes);
    }

    private void showAccountChooser() {
        if (mCredential == null) {
            mCredential = makeCredential();
        }
        startActivityForResult(mCredential.newChooseAccountIntent(), REQUEST_ACCOUNT_CHOOSER);
    }

    private void listDriveFiles() {
        if (mDrive == null) {
            mDrive = buildDrive();
        }
        new ListDriveFilesTask().execute();
    }

    private void showPlusUser() {
        if (mPlus == null) {
            mPlus = buildPlus();
        }
        new ShowPlusUserTask().execute();
    }

    @NonNull
    private Drive buildDrive() {
        HttpTransport transport = AndroidHttp.newCompatibleTransport();
        GsonFactory factory = new GsonFactory();
        return new Drive.Builder(transport, factory, mCredential).build();
    }

    @NonNull
    private Plus buildPlus() {
        HttpTransport transport = AndroidHttp.newCompatibleTransport();
        GsonFactory factory = new GsonFactory();
        return new Plus.Builder(transport, factory, mCredential).build();
    }

    /**
     *  Initialize a custom log class that outputs both to in-app targets and logcat.
     */
    private void initializeLogging() {
        // Wraps Android's native log framework.
        LogWrapper logWrapper = new LogWrapper();
        // Using Log, front-end to the logging chain, emulates android.util.log method signatures.
        Log.setLogNode(logWrapper);
        // Filter strips out everything except the message text.
        MessageOnlyLogFilter msgFilter = new MessageOnlyLogFilter();
        logWrapper.setNext(msgFilter);
        // On screen logging via a customized TextView.
        LogView logView = (LogView) findViewById(R.id.logView);
        logView.setTextAppearance(this, R.style.Log);
        logView.setBackgroundColor(Color.WHITE);
        msgFilter.setNext(logView);
        Log.i(TAG, "Ready");
    }

    private class ListDriveFilesTask extends AsyncTask<Void, Void, Void> {
        private Intent mIntent;
        private List<File> mList;

        @Override
        protected Void doInBackground(Void... unused) {
            mList = new ArrayList<>();
            try {
                Drive.Files.List request = mDrive.files().list();
                request.setQ("mimeType = 'application/vnd.google-apps.document'");
                do {
                    FileList files = request.execute();
                    mList.addAll(files.getItems());
                    request.setPageToken(files.getNextPageToken());
                } while (request.getPageToken() != null && request.getPageToken().length() > 0);
            } catch (UserRecoverableAuthIOException e) {
                mIntent = e.getIntent();
            } catch (IOException e) {
                Log.e(TAG, e.getMessage(), e);
            }
            return null;
        }

        @Override
        protected void onPostExecute(Void unused) {
            if (mIntent != null) {
                startActivityForResult(mIntent, REQUEST_AUTHORIZATION_FROM_DRIVE);
            } else {
                for (File file : mList) {
                    Log.d(TAG, file.getTitle() + " " + file.getOwnerNames() + "");
                }
            }
        }
    }

    private class ShowPlusUserTask extends AsyncTask<Void, Void, Void> {
        private Person mPerson;
        private Intent mIntent;

        @Override
        protected Void doInBackground(Void... unused) {
            try {
                mPerson = mPlus.people().get("me").execute();
            } catch (UserRecoverableAuthIOException e) {
                mIntent = e.getIntent();
            } catch (IOException e) {
                Log.e(TAG, e.getMessage(), e);
            }
            return null;
        }

        @Override
        protected void onPostExecute(Void unused) {
            if (mIntent != null) {
                startActivityForResult(mIntent, REQUEST_AUTHORIZATION_FROM_PLUS);
            } else {
                Log.d(TAG, "Display Name:" + mPerson.getDisplayName());
                Log.d(TAG, "Birthday:" + mPerson.getBirthday());
                Log.d(TAG, "Gender:" + mPerson.getGender());
                Log.d(TAG, "toString():" + mPerson.toString());
            }
        }
    }
}
